"""
allows you to modify any constant from the command line.

use lazyopt.apply_all() to apply values from command line.

https://github.com/neyer/lazyopt

"""
__version__ = '1.0.0'

import inspect
import os
import sys

class ConfigurationError(Exception):
  """Invalid option specified."""


def get_as_number(value):
  """Return str `value` as a number if possible. Return None otherwise."""
  if not value: return None
  if value.count('.') == 1:
    return float(value)
  else:
    return int(value)

def cast_value_str(value):
  "Interpret str `value` as whatever datatype makes sense"

  # first see if it's one of these constants
  hard_coded = { 'None' : None, 'True' : True, 'False' : False }
  try:
    return hard_coded[value]
  except KeyError:
    # if it's not a constant see if it's a number
    try:
      return get_as_number(value)
    except ValueError:
      # it must be a string
      return value
  

def get_argv_bindings(argv):
  """Parse `argv` and return dict of new bindings."""
  results = {}
  this_arg_name = None

  for arg in argv:
    #if we have the name of an argument,
    #we are waiting for a variable
    if this_arg_name:
      #if the next arg is another name, this_arg_name is a flag
      if arg.find('--') == 0:
        value = 'True'
        name = this_arg_name
        this_arg_name = arg[2:]
      else:
        value = arg
        name = this_arg_name
        this_arg_name = None

      #make sure they haven't given the same arg twice
      if name in results:
        raise ConfigurationError("duplicate arg %s" % name)
      else:
        # store the binding
        results[name] = cast_value_str(value)
    else:
      #check to see if this option is an arg name
      if arg.find('--') == 0:
        this_arg_name = arg[2:]
        #check the next arg for the value
      else: pass
        # this is a position argument. just ignore it.

  # we looped through all the args and have one left
  # so that must be a boolean flag
  if this_arg_name:
    if this_arg_name in results:
      raise ConfigurationError("duplicate arg %s" % this_arg_name)
    else:
      results[this_arg_name] = True

  return results


def get_module_and_var_name(var_name):
  "Convert an option name to a module,variable pair."
  parts = var_name.split('.')
  module_name = ('.'.join(parts[:-1])).replace('-','_')
  var_name = parts[-1].replace('-','_')

  return module_name, var_name


def apply_binding(module_name, var_name, value):
  "Set module `module_name` variable `var_name` to `value`."

  __import__(module_name, globals(), locals(), [], 0)
  module = sys.modules.get(module_name)

  if hasattr(module, var_name):
    setattr(module, var_name, value)
  else:
    msg = 'module %s has no value %s to confgure with value %s.'
    msg = msg % (module_name, var_name, value)
    raise ConfigurationError(msg)

def get_caller_module():
  "get the name of the module in which this function was called."
  stack = inspect.currentframe()
  return inspect.getmodule(stack.f_back.f_back.f_code)

def apply_all(argv=sys.argv):
  bindings = get_argv_bindings(argv)
  # figure out who called into this frame so args without module names work
  caller_module = get_caller_module().__name__

  for name, value in bindings.items():
      module_name, var_name = get_module_and_var_name(name)
      if not module_name:
          module_name = caller_module
      apply_binding(module_name, var_name, value)




import lazyopt
import unittest


class LazyoptTest(unittest.TestCase):

  def test_cast_value(self):
    "Ensure that string values are cast to the appropriate types."

    test_pairs = {
        "foo" : "foo",
        "1" : 1,
         "0" : 0,
        "1.5" : 1.5,
        "0.0.1" : "0.0.1",
        "False" : False,
        "None" :  None,
        "True" : True
    }

    for k, v in test_pairs.items():
      self.assertEqual(lazyopt.cast_value_str(k),v)

  def test_get_argv_bindings(self):
    "Test that argv strings are properly parsed."
    args = ["--bool-flag", 
            "--foo-int", "5",
            "--bar-float", "4.5",
            "ignore-me",
            "ignore-me-too",
            "--baz-str", "b.b.b",
            "--non-str", "None",
            "--is-false", "False",
            "--end-flag"]


    bindings = lazyopt.get_argv_bindings(args)

    self.assertEqual(bindings["bool-flag"], True)
    self.assertEqual(bindings["foo-int"], 5)
    self.assertEqual(bindings["bar-float"], 4.5)
    self.assertEqual(bindings["baz-str"], "b.b.b")
    self.assertEqual(bindings["non-str"], None)
    self.assertEqual(bindings["is-false"], False)
    self.assertEqual(bindings["end-flag"], True)

    # now try a duplicate config
    args = [ "--double-flag", "--double-flag" ]

    with self.assertRaises(lazyopt.ConfigurationError):
      lazyopt.get_argv_bindings(args)


  def test_get_module_and_var_name(self):

    tests = [
      ["foo.bar", "foo", "bar"],
      ["foo.bar.baz", "foo.bar", "baz"],
      ["foo.bar-baz.bat", "foo.bar_baz", "bat"],
      ["foo-bar.baz-bat", "foo_bar", "baz_bat"],
    ]
    for pair in tests:
        res = lazyopt.get_module_and_var_name(pair[0])
        self.assertEqual(res[0], pair[1])
        self.assertEqual(res[1], pair[2])


  def test_apply_binding(self):
    "Make sure bindings can be applied to a module"

    lazyopt.apply_binding("lazyopt.fake_module","VAL_IS_2", 3)
    lazyopt.apply_binding("lazyopt.fake_module","VAL_IS_FOO", "bar")

    self.assertEqual(lazyopt.fake_module.VAL_IS_2, 3)
    self.assertEqual(lazyopt.fake_module.VAL_IS_FOO, "bar")


    with self.assertRaises(lazyopt.ConfigurationError):
      lazyopt.apply_binding("lazyopt.fake_module","PANTS_MAN", "totenhosen")

  def test_get_caller_module(self):

    
    def spurious_function():
      return lazyopt.get_caller_module().__name__
    
    module_name = spurious_function()
    self.assertEqual(module_name, 'lazyopt.tests')








from distutils.core import setup
PACKAGE = "lazyopt"
NAME = "lazyopt"
DESCRIPTION = "the lazy coder's option parser"
AUTHOR = "mark neyer"
AUTHOR_EMAIL = "mneyer@gmail.com"
URL = "https://github.com/neyer/lazyopt"
VERSION = __import__(PACKAGE).__version__

setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    license="BSD",
    url=URL,
    packages=[PACKAGE]
)
